"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DB = void 0;
const app_1 = require("firebase-admin/app");
const firestore_1 = require("firebase-admin/firestore");
class DB {
    constructor(config) {
        if (!config) {
            throw new Error("No config provided.");
        }
        this.config = typeof config == "function" ? config() : config;
        this.app = (0, app_1.initializeApp)({
            credential: (0, app_1.cert)(this.config),
        });
        this.db = (0, firestore_1.getFirestore)(this.app);
    }
    addData(collectionName, documentName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!collectionName || !documentName || !data) {
                throw new Error("No collectionName or documentName or data was provided!");
            }
            const colName = typeof collectionName === "function" ? collectionName() : collectionName;
            const docName = typeof documentName === "function" ? documentName() : documentName;
            const docData = typeof data === "function" ? data() : data;
            try {
                const dataRef = this.db.collection(colName).doc(docName);
                yield dataRef.set(docData);
                return true;
            }
            catch (error) {
                console.error(`There was an error adding data to the collection "${colName}"! error: ${error}`);
                return false;
            }
        });
    }
    getData(collectionName, documentName) {
        return __awaiter(this, void 0, void 0, function* () {
            const colName = typeof collectionName === "function" ? collectionName() : collectionName;
            const docName = typeof documentName === "function" ? documentName() : documentName;
            try {
                const dataRef = this.db.collection(colName).doc(docName);
                const doc = yield dataRef.get();
                if (!doc.exists) {
                    return false;
                }
                else {
                    return doc.data();
                }
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
    deleteDocument(collectionName, documentName) {
        return __awaiter(this, void 0, void 0, function* () {
            const colName = typeof collectionName === "function" ? collectionName() : collectionName;
            const docName = typeof documentName === "function" ? documentName() : documentName;
            try {
                yield this.db.collection(colName).doc(docName).delete();
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
    deleteBatches(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const snapshot = yield query.get();
            const batchSize = snapshot.size;
            if (batchSize === 0) {
                return true;
            }
            const batch = this.db.batch();
            snapshot.docs.forEach((doc) => {
                batch.delete(doc.ref);
            });
            yield batch.commit();
            process.nextTick(() => {
                this.deleteBatches(query);
            });
        });
    }
    deleteCollection(collectionName, batchSize = 5) {
        return __awaiter(this, void 0, void 0, function* () {
            const colName = typeof collectionName === "function" ? collectionName() : collectionName;
            const batch = typeof batchSize === "function" ? batchSize() : batchSize;
            try {
                const collectionRef = yield this.db.collection(colName);
                const query = yield collectionRef.orderBy("__name__").limit(batch);
                yield this.deleteBatches(query);
                return true;
            }
            catch (error) {
                console.error(error);
                return false;
            }
        });
    }
}
exports.DB = DB;
