import { initializeApp, cert } from "firebase-admin/app";
import { getFirestore, } from "firebase-admin/firestore";
export class DB {
    config;
    app;
    db;
    constructor(config) {
        if (!config) {
            throw new Error("No config provided.");
        }
        this.config = typeof config == "function" ? config() : config;
        this.app = initializeApp({
            credential: cert(this.config),
        });
        this.db = getFirestore(this.app);
    }
    async addData(collectionName, documentName, data) {
        if (!collectionName || !documentName || !data) {
            throw new Error("No collectionName or documentName or data was provided!");
        }
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        const docData = typeof data === "function" ? data() : data;
        try {
            const dataRef = this.db.collection(colName).doc(docName);
            await dataRef.set(docData);
            return true;
        }
        catch (error) {
            console.error(`There was an error adding data to the collection "${colName}"! error: ${error}`);
            return false;
        }
    }
    async getData(collectionName, documentName) {
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        try {
            const dataRef = this.db.collection(colName).doc(docName);
            const doc = await dataRef.get();
            if (!doc.exists) {
                return false;
            }
            else {
                return doc.data();
            }
        }
        catch (error) {
            console.error(error);
            return false;
        }
    }
    async deleteDocument(collectionName, documentName) {
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        try {
            await this.db.collection(colName).doc(docName).delete();
            return true;
        }
        catch (error) {
            console.error(error);
            return false;
        }
    }
    async deleteBatches(query) {
        const snapshot = await query.get();
        const batchSize = snapshot.size;
        if (batchSize === 0) {
            return true;
        }
        const batch = this.db.batch();
        snapshot.docs.forEach((doc) => {
            batch.delete(doc.ref);
        });
        await batch.commit();
        process.nextTick(() => {
            this.deleteBatches(query);
        });
    }
    async deleteCollection(collectionName, batchSize = 5) {
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const batch = typeof batchSize === "function" ? batchSize() : batchSize;
        try {
            const collectionRef = await this.db.collection(colName);
            const query = await collectionRef.orderBy("__name__").limit(batch);
            await this.deleteBatches(query);
            return true;
        }
        catch (error) {
            console.error(error);
            return false;
        }
    }
}
